#include <AfxExt.h>
#define INITGUID
#include <d3drm.h>
#include <d3drmwin.h>
#include "Direct3DWin.h"

BEGIN_MESSAGE_MAP(Direct3DWin, CFrameWnd)
	ON_WM_CREATE()
	ON_WM_PAINT()
	ON_WM_TIMER()
	ON_WM_DESTROY()
END_MESSAGE_MAP()

Direct3DWin::Direct3DWin()
{
	m_D3DOK = FALSE;
	m_pD3D = NULL;
	m_pDevice = NULL;
	m_pViewport = NULL;
	m_pParentFrame = NULL;
	m_pEye = NULL;
	m_pMeshBuilder = NULL;
	m_pClipper = NULL;
    Create(NULL, "Direct3DApp");
}

Direct3DWin::~Direct3DWin()
{
}

int Direct3DWin::OnCreate(LPCREATESTRUCT)
{
	HRESULT result = Direct3DRMCreate(&m_pD3D);
	if (result != D3DRM_OK)
		return 0;

	result = DirectDrawCreateClipper(0, &m_pClipper, 0);
	if (result != DD_OK)
		return 0;
	result = m_pClipper->SetHWnd(0, m_hWnd);
	if (result != DD_OK)
		return 0;

	RECT rect;
	::GetClientRect(m_hWnd, &rect);
	result = m_pD3D->CreateDeviceFromClipper(m_pClipper,
		NULL, rect.right, rect.bottom, &m_pDevice);
	if (result != D3DRM_OK)
		return 0;
	m_pDevice->SetQuality(D3DRMRENDER_GOURAUD);

	m_pD3D->CreateFrame(0, &m_pParentFrame);
	if (result != D3DRM_OK)
		return 0;

	BOOL succeeded = MakeScene();
	if (!succeeded)
		return 0;

	m_D3DOK = TRUE;

	SetTimer(1, 100, NULL);

	return 0;
}

BOOL Direct3DWin::MakeScene()
{
	BOOL succeeded = CreateMesh();
	if (!succeeded)
		return FALSE;

	succeeded = CreateViewport();
	if (!succeeded)
		return FALSE;

	succeeded = CreateLight();
	if (!succeeded)
		return FALSE;

	return TRUE;
}

BOOL Direct3DWin::CreateMesh()
{
	HRESULT result =
		m_pD3D->CreateMeshBuilder(&m_pMeshBuilder);
	if (result != D3DRM_OK)
		return FALSE;

	result = m_pMeshBuilder->Load("model.x",
		NULL, D3DRMLOAD_FROMFILE, NULL, NULL );
	if (result != D3DRM_OK)
	{
		MessageBox("MODEL.X file missing. Please place\n\
a Direct3D X file with this name\nin the Direct3DApp folder.");
		return FALSE;
	}

	LPDIRECT3DRMFRAME pFrame;
	result =
		m_pD3D->CreateFrame(m_pParentFrame, &pFrame);
	if (result != D3DRM_OK)
		return FALSE;
	pFrame->AddVisual(m_pMeshBuilder);
	pFrame->SetRotation(m_pParentFrame, D3DVALUE(0.0),
		D3DVALUE(1.0), D3DVALUE(0.0), D3DVALUE(0.05));
	pFrame->Release();
	pFrame = NULL;

	return TRUE;
}

BOOL Direct3DWin::CreateViewport()
{
	HRESULT result = m_pD3D->
		CreateFrame(m_pParentFrame, &m_pEye);
	if (result != D3DRM_OK)
		return FALSE;
	m_pEye->SetPosition(m_pParentFrame, D3DVALUE(0),
		D3DVALUE(0), D3DVALUE(-40.0));
	result = m_pD3D->CreateViewport(m_pDevice, m_pEye,
		0, 0, m_pDevice->GetWidth(),
		m_pDevice->GetHeight(), &m_pViewport);
	if (result != D3DRM_OK)
		return FALSE;

	return TRUE;
}

BOOL Direct3DWin::CreateLight()
{
	LPDIRECT3DRMLIGHT pLight;
	HRESULT result = m_pD3D->
		CreateLightRGB(D3DRMLIGHT_DIRECTIONAL,
		D3DVALUE(1.0), D3DVALUE(1.0), D3DVALUE(1.0),
		&pLight);
	if (result != D3DRM_OK)
		return FALSE;

	LPDIRECT3DRMFRAME pLightFrame;
	result = m_pD3D->CreateFrame(m_pParentFrame,
		&pLightFrame );
	if (result != D3DRM_OK)
		return FALSE;

	pLightFrame->AddLight(pLight);
	pLightFrame->SetOrientation(m_pParentFrame,
		D3DVALUE(0.0), D3DVALUE(-1.0), D3DVALUE(1.0),
		D3DVALUE(0.0), D3DVALUE(1.0), D3DVALUE(0.0));
	pLight->Release();
	pLight = NULL;
	pLightFrame->Release();
	pLightFrame = NULL;

	return TRUE;
}

void Direct3DWin::OnPaint() 
{
	BOOL repaint = GetUpdateRect(NULL);
	if (!repaint)
		return;

	PAINTSTRUCT pstruct;
	BeginPaint(&pstruct);

	if (m_D3DOK) 
	{
		LPDIRECT3DRMWINDEVICE pWndDevice;
		m_pDevice->QueryInterface(
			IID_IDirect3DRMWinDevice,
			(void**)&pWndDevice);
        pWndDevice->HandlePaint(pstruct.hdc);
		pWndDevice->Release();
	}
	else
		::TextOut(pstruct.hdc, 20, 20,
		    "Direct3D failed to start.", 25);

	EndPaint(&pstruct);
}

void Direct3DWin::OnTimer(UINT nIDEvent) 
{
	if (m_D3DOK)
	    m_pD3D->Tick(D3DVALUE(1));
}

void Direct3DWin::OnDestroy()
{
	if (m_pMeshBuilder)
		m_pMeshBuilder->Release();

	if (m_pEye)
		m_pEye->Release();

	if (m_pViewport)
		m_pViewport->Release();

	if (m_pParentFrame)
		m_pParentFrame->Release();

	if (m_pDevice)
		m_pDevice->Release();

	if (m_pD3D)
		m_pD3D->Release();

	if (m_pClipper)
		m_pClipper->Release();
}

